@@ -1,19 +1,5 @@
 # texttable - module for creating simple ASCII tables
-# Copyright (C) 2003-2015 Gerome Fournier <jef(at)foutaise.org>
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or (at your option) any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+# Copyright (C) 2003-2018 Gerome Fournier <jef(at)foutaise.org>
 
 """module for creating simple ASCII tables
 
@@ -75,8 +61,8 @@ from __future__ import division
 __all__ = ["Texttable", "ArraySizeError"]
 
 __author__ = 'Gerome Fournier <jef(at)foutaise.org>'
-__license__ = 'LGPL'
-__version__ = '1.4.0'
+__license__ = 'MIT'
+__version__ = '1.6.1'
 __credits__ = """\
 Jeff Kowalczyk:
     - textwrap improved import
@@ -124,6 +110,26 @@ except ImportError:
         sys.stderr.write("Can't import textwrap module!\n")
         raise
 
+# define a function to calculate the rendering width of a unicode character
+# - use wcwidth if available
+# - fallback to unicodedata information otherwise
+try:
+    import wcwidth
+    def uchar_width(c):
+        """Return the rendering width of a unicode character
+        """
+        return max(0, wcwidth.wcwidth(c))
+except ImportError:
+    def uchar_width(c):
+        """Return the rendering width of a unicode character
+        """
+        if unicodedata.east_asian_width(c) in 'WF':
+            return 2
+        elif unicodedata.combining(c):
+            return 0
+        else:
+            return 1
+
 from functools import reduce
 
 if sys.version_info >= (3, 0):
@@ -153,9 +159,7 @@ def len(iterable):
     """Redefining len here so it will be able to work with non-ASCII characters
     """
     if isinstance(iterable, bytes_type) or isinstance(iterable, unicode_type):
-        unicode_data = obj2unicode(iterable)
-        w = unicodedata.east_asian_width
-        return sum([w(c) in 'WF' and 2 or (0 if unicodedata.combining(c) else 1) for c in unicode_data])
+        return sum([uchar_width(c) for c in obj2unicode(iterable)])
     else:
         return iterable.__len__()
 
@@ -191,9 +195,7 @@ class Texttable:
         - if set to 0, size is unlimited, therefore cells won't be wrapped
         """
 
-        if max_width <= 0:
-            max_width = False
-        self._max_width = max_width
+        self.set_max_width(max_width)
         self._precision = 3
 
         self._deco = Texttable.VLINES | Texttable.HLINES | Texttable.BORDER | \
@@ -211,6 +213,16 @@ class Texttable:
         self._row_size = None
         self._header = []
         self._rows = []
+        return self
+
+    def set_max_width(self, max_width):
+        """Set the maximum width of the table
+
+        - max_width is an integer, specifying the maximum width of the table
+        - if set to 0, size is unlimited, therefore cells won't be wrapped
+        """
+        self._max_width = max_width if max_width > 0 else False
+        return self
 
     def set_chars(self, array):
         """Set the characters used to draw lines between rows and columns
@@ -229,6 +241,7 @@ class Texttable:
         array = [ x[:1] for x in [ str(s) for s in array ] ]
         (self._char_horiz, self._char_vert,
             self._char_corner, self._char_header) = array
+        return self
 
     def set_deco(self, deco):
         """Set the table decoration
@@ -248,6 +261,7 @@ class Texttable:
         """
 
         self._deco = deco
+        return self
 
     def set_header_align(self, array):
         """Set the desired header alignment
@@ -261,6 +275,7 @@ class Texttable:
 
         self._check_row_size(array)
         self._header_align = array
+        return self
 
     def set_cols_align(self, array):
         """Set the desired columns alignment
@@ -274,6 +289,7 @@ class Texttable:
 
         self._check_row_size(array)
         self._align = array
+        return self
 
     def set_cols_valign(self, array):
         """Set the desired columns vertical alignment
@@ -287,6 +303,7 @@ class Texttable:
 
         self._check_row_size(array)
         self._valign = array
+        return self
 
     def set_cols_dtype(self, array):
         """Set the desired columns datatype for the cols.
@@ -306,6 +323,7 @@ class Texttable:
 
         self._check_row_size(array)
         self._dtype = array
+        return self
 
     def set_cols_width(self, array):
         """Set the desired columns width
@@ -325,6 +343,7 @@ class Texttable:
             sys.stderr.write("Wrong argument in column width specification\n")
             raise
         self._width = array
+        return self
 
     def set_precision(self, width):
         """Set the desired precision for float/exponential formats
@@ -337,6 +356,7 @@ class Texttable:
         if not type(width) is int or width < 0:
             raise ValueError('width must be an integer greater then 0')
         self._precision = width
+        return self
 
     def header(self, array):
         """Specify the header of the table
@@ -344,6 +364,7 @@ class Texttable:
 
         self._check_row_size(array)
         self._header = list(map(obj2unicode, array))
+        return self
 
     def add_row(self, array):
         """Add a row in the rows stack
@@ -360,6 +381,7 @@ class Texttable:
         for i, x in enumerate(array):
             cells.append(self._str(i, x))
         self._rows.append(cells)
+        return self
 
     def add_rows(self, rows, header=True):
         """Add several rows in the rows stack
@@ -380,6 +402,7 @@ class Texttable:
                 rows = rows[1:]
         for row in rows:
             self.add_row(row)
+        return self
 
     def draw(self):
         """Draw the table
